/*********************************************
* UART interface without interrupt
* Author: Guido Socher, Copyright: GPL
* Copyright: GPL
**********************************************/

#include "uart.h"


void uart_init(void)
{

  uint32_t baud= 115200;

  uint16_t ubrr;

  if (baud> 57600)
  {
    baud= baud>>1;
    ubrr= (F_CPU/16/baud);
    ubrr--;
    UCSR0A |= 1<<U2X0;                                  // Baudrate verdoppeln
  }
  else
  {
    ubrr= (F_CPU/16/baud-1);
  }
  UBRR0H = (unsigned char)(ubrr>>8);                    // Baudrate setzen
  UBRR0L = (unsigned char)ubrr;

#if defined (__AVR_ATmega328P__) || defined (__AVR_ATmega168__) || defined (__AVR_ATmega88__)

    UCSR0B = (1<<RXEN0)|(1<<TXEN0);                       // Transmitter und Receiver enable
    UCSR0C = (3<<UCSZ00);                                 // 8 Datenbit, 1 Stopbit

#else

    // ATmega8
    UCSR0B |= (1<<RXEN0) | (1<<TXEN0);
    UCSRC  = (1<<URSEL) | (1<<UCSZ01) | (1<<UCSZ00);

#endif
}


/* send one character to the rs232 */
void uart_sendchar(char c)
{
        /* wait for empty transmit buffer */
        while (!(UCSRA & (1<<UDRE)));
        UDR=c;
}
/* send string to the rs232 */
void uart_sendstr(char *s)
{
        while (*s){
                uart_sendchar(*s);
                s++;
        }
}

void uart_sendstr_p(const char *progmem_s)
/* print string from program memory on rs232 */
{
        char c;
        while ((c = pgm_read_byte(progmem_s++))) {
                uart_sendchar(c);
        }

}

/* get a byte from rs232
* this function does a blocking read */
unsigned char uart_getchar(unsigned char kickwd)
{
//  while( !(UCSRA & (1<<RXC)) )
  while( !(UCSRA & (1<<RXC)) && !(bridge) )
  {
    // we can not aford a watchdog timeout
    // because this is a blocking function
    if (kickwd)
    {
      wd_kick();
    }
  }
  return(UDR);
}

/* read and discard any data in the receive buffer */
void uart_flushRXbuf(void)
{
        unsigned char tmp;
        while(UCSRA & (1<<RXC)){
                tmp=UDR;
        }
}

  /* --------------------------------------------------
      testen, ob ein Zeichen auf der Schnittstelle
      ansteht
     -------------------------------------------------- */

  unsigned char uart_ischar( void )
  {
    return (UCSRA & (1 << RXC));
  }
